@@ -7,18 +7,13 @@ Outputs are chemical species.
 
 Notes:
 1. Reaction enzymes are identified by the SBML reaction ID.
+2. The Jacobian is only recalculated if there is a change in timepoint
 """
 
-"""
-TO DO:
-
-1. Plot difference between time jacoabian at reference vs. Current.
-2. Plot TOTAL residual SSQ vs. jacobian difference
-"""
 
 from controlSBML.make_roadrunner import makeRoadrunner
-from controlSBML import msgs
 import controlSBML as ctl
+from controlSBML import util
 
 import control
 import numpy as np
@@ -31,6 +26,7 @@ END_TIME = 5  # Default endtime
 POINTS_PER_TIME = 10
 TIME = "time"
 IS_DEBUG = False
+TIMEPOINT_NULL = 1
 
 
 cleanIt = lambda n: n if n[0] != "[" else n[1:-1]
@@ -50,22 +46,18 @@ class ControlBase(object):
         is_reduced: bool
             construct a reduced model so that the A matrix is nonsingular
         """
-        def calcNames(superset_names, subset_names, superset_name, subset_name):
-            if subset_names is None:
-                return list(superset_names)
-            else:
-                return subset_names
         self.is_reduced = is_reduced
         # Iinitial model calculations
         self.model_reference = model_reference
         self.roadrunner = makeRoadrunner(self.model_reference)
+        self._jacobian_timepoint = TIMEPOINT_NULL
+        self._jacobian_df = None
         # Set defaults
         self.species_names = list(
               self.roadrunner.getFullStoichiometryMatrix().rownames)
         self.depeendent_names = list(
               set(self.species_names).symmetric_difference(self.state_names))
         if not set(self.state_names) <= set(self.species_names):
-            import pdb; pdb.set_trace()
             raise RuntimeError("State name is not a species name.")
         self.full_stoichiometry_df, self.reduced_stoichiometry_df  \
               = self._makeStoichiometryDF()
@@ -78,8 +70,9 @@ class ControlBase(object):
             self.input_names = input_names
         self.input_names = self._sortList(self.reaction_names, self.input_names)
         self.num_input = len(self.input_names)
-        self.output_names = calcNames(self.species_names, output_names,
-              "states", "outputs")
+        if output_names is None:
+            output_names = self.species_names
+        self.output_names = output_names
         self.output_names = self._sortList(self.species_names, self.output_names)
         self.num_output = len(self.output_names)
         # Other calculations
@@ -107,11 +100,6 @@ class ControlBase(object):
     def C_df(self):
         return self._makeCDF()
 
-    @staticmethod
-    def _makeDF(mat):
-        df = pd.DataFrame(mat, columns=mat.colnames, index=mat.rownames)
-        return df
-
     def _makeStoichiometryDF(self):
         """
         Creates the reduced stoichiometry matrix and the auxiliary matrix
@@ -122,9 +110,9 @@ class ControlBase(object):
         DataFrame - reduced stoichiometry matrix
         """
         #
-        reduced_stoichiometry_df = self._makeDF(
+        reduced_stoichiometry_df = util.mat2DF(
               self.roadrunner.getReducedStoichiometryMatrix())
-        full_stoichiometry_df = self._makeDF(
+        full_stoichiometry_df = util.mat2DF(
               self.roadrunner.getFullStoichiometryMatrix())
         return full_stoichiometry_df, reduced_stoichiometry_df
 
@@ -140,11 +128,15 @@ class ControlBase(object):
         state_names = self.state_names
         C_df = pd.DataFrame(np.eye(self.num_state), columns=state_names,
               index=state_names)
-        L0 = self.roadrunner.getL0Matrix()
-        if len(L0) > 0:
-            L0_df = pd.DataFrame(L0, columns=L0.colnames, index=L0.rownames)
-            C_df = pd.concat([C_df, L0_df], axis=0)
+        if self.is_reduced:
+            L0 = self.roadrunner.getL0Matrix()
+            if len(L0) > 0:
+                L0_df = pd.DataFrame(L0, columns=L0.colnames, index=L0.rownames)
+                C_df = pd.concat([C_df, L0_df], axis=0)
         C_df = C_df.loc[self.output_names, :]
+        values = C_df.values.flatten()
+        if any([np.isnan(v) for v in values]):
+            import pdb; pdb.set_trace()
         return C_df
 
     @property
@@ -154,7 +146,7 @@ class ControlBase(object):
 
         Parameters
         ----------
-
+:
         Returns
         -------
         dict
@@ -166,8 +158,8 @@ class ControlBase(object):
                self.roadrunner.getBoundarySpeciesIds(),
                self.roadrunner.getGlobalParameterIds(),
                ]:
-            for id in id_lst:
-                 dct[id] = self.get(id)
+            for idx in id_lst:
+                 dct[idx] = self.get(idx)
         return dct
 
     @property
@@ -179,7 +171,8 @@ class ControlBase(object):
         -------
         np.ndarray: N X 1
         """
-        return self._makeSer(self.state_names)
+        ser = util.makeRoadrunnerSer(self.roadrunner, self.state_names)
+        return ser
 
     @property
     def output_ser(self):
@@ -190,27 +183,21 @@ class ControlBase(object):
         -------
         np.ndarray: N X 1
         """
-        return self._makeSer(self.output_names)
-
-    def _makeSer(self, names):
-        """
-        Contructs a Series for the names.
-
-        Returns
-        -------
-        pd.Series
-        """
-        return pd.Series(list(self.get(names).values()), index=names)
+        return util.makeSer(self.roadrunner, self.output_names)
 
     @property
     def jacobian_df(self):
         """
+        Calculates the Jacobian, or a reduced Jacobian if the option is selected.
+        Improves efficiency by using a previous calculation if the time has not changed.
+
         Returns
         -------
         pd.DataFrame, species_names
         """
-        # FIXME: Use reduced stoichiometry matrix once it's clear how
-        #        to calculate dependent spcies
+        if np.isclose(self.getTime(), self._jacobian_timepoint):
+            if self._jacobian_df is not None:
+                return self._jacobian_df
         if self.is_reduced:
             current_bool = self.roadrunner.conservedMoietyAnalysis
             self.roadrunner.conservedMoietyAnalysis = True
@@ -221,8 +208,9 @@ class ControlBase(object):
         if len(jacobian_mat.rownames) != len(jacobian_mat.colnames):
             raise RuntimeError("Jacobian is not square!")
         names = list(jacobian_mat.colnames)
-        jacobian_df = pd.DataFrame(jacobian_mat, columns=names, index=names)
-        return jacobian_df
+        self._jacobian_df = pd.DataFrame(jacobian_mat, columns=names, index=names)
+        self._jacobian_timepoint = self.getTime()
+        return self._jacobian_df
 
     @property
     def state_names(self):
@@ -243,6 +231,7 @@ class ControlBase(object):
 
     def setTime(self, time):
         self.roadrunner.reset()
+        self._jacobian_timepoint = TIMEPOINT_NULL
         if time > 0.01:
             _ = self.roadrunner.simulate(0.0, time)
 
@@ -264,7 +253,7 @@ class ControlBase(object):
     def getTime(self):
         """
         Gets current simulation time.
-        
+
         Returns
         -------
         float
@@ -332,11 +321,9 @@ class ControlBase(object):
         -------
         object/dict
         """
-        if isinstance(names, str):
-            return self.roadrunner[names]
         if names is None:
             names = self.roadrunner.keys()
-        return {n: self.roadrunner[n] for n in names}
+        return util.getRoadrunnerValue(self.roadrunner, names)
 
     def set(self, name_dct):
         """
@@ -348,10 +335,7 @@ class ControlBase(object):
             key: str
             value: value
         """
-        for name, value in name_dct.items():
-            if isinstance(value, int):
-                value = float(value)
-            self.roadrunner[name] = value
+        util.setRoadrunnerValue(self.roadrunner, name_dct)
 
     @staticmethod
     def _sortList(super_lst, sub_lst):
@@ -442,20 +426,24 @@ class ControlBase(object):
                 ncol = np.shape(B_mat)[1]
             D_mat = np.repeat(0, nrow*ncol)
             D_mat = np.reshape(D_mat, (nrow, ncol))
+        try:
+            ss = control.StateSpace(A_mat, B_mat, C_mat, D_mat)
+        except:
+            import pdb; pdb.set_trace()
         return control.StateSpace(A_mat, B_mat, C_mat, D_mat)
 
     def makeNonlinearIOSystem(self, name, effector_dct=None):
         """
         Creates an object that can be used in connections with the
         control package.
- 
+
         Parameters
         ----------
         name: str (name of the system)
         effector_dct: dict (maps reaction inputs to roadrunner muteables)
             key: str (input name)
             value: str (name of roadrunner muteable)
-        
+
         Returns
         -------
         controlSBML.NonelinearIOSystem
